/*
 * =====================================================================================================
 *
 * 	errors.h
 *
 * 	Copyright (c) 2010 Smartsoft. All rights reserved.
 *
 * 	Last committed on: 	Jan 16 2010
 * 	Author: 			Mehmet Altay Ural
 *
 * =====================================================================================================
 */
 
#ifndef _ERROR_H_
#define _ERROR_H_

// Error Codes

#define ERR_BASE					0x100

// GENERAL

#define PROCESS_OK 					ERR_BASE
#define INVALID_ARG					ERR_BASE + 0x1

// FILE OPERATION BASED ERRORS

#define FILE_ERR_BASE				ERR_BASE + 0x200
#define FILE_OPEN_FAILED			FILE_ERR_BASE + 0x1
#define FILE_OPERATION_FAILED		FILE_ERR_BASE + 0x2	// write || read

// MEMORY OPERATION BASED ERRORS

#define MEMORY_ERR_BASE				FILE_ERR_BASE + 0x50
#define	MEMORY_NOT_ENOUGH			MEMORY_ERR_BASE + 0x1


// SMARTCARD BASED ERRORS

#define SMARTCARD_ERR_BASE			ERR_BASE + 0x300
#define SMARTCARD_OK                SMARTCARD_ERR_BASE + 0x1
#define SMARTCARD_FAIL				SMARTCARD_ERR_BASE + 0x2
#define SMARTCARD_INIT_FAILED		SMARTCARD_ERR_BASE + 0x3
#define FALLBACK_SITUATION			SMARTCARD_ERR_BASE + 0x4

// 
#define TTL_ERR_BASE				ERR_BASE + 0x400

// EMV BASED ERRORS

#define EMV_ERR_BASE						ERR_BASE + 0x500
#define EMV_PARSING_TAGS_FAILED				EMV_ERR_BASE + 0x1
#define EMV_DUPLICATE_CARD_DATA_ELEMENT		EMV_ERR_BASE + 0x2
#define EMV_DATA_FORMAT_INCORRECT 			EMV_ERR_BASE + 0x3
#define EMV_NO_TERM_APP						EMV_ERR_BASE + 0x4
#define EMV_NO_MATCHING_APP 				EMV_ERR_BASE + 0x5
#define EMV_MISSING_MANDATORY_OBJECT 		EMV_ERR_BASE + 0x6
#define EMV_APP_SELECTION_RETRY				EMV_ERR_BASE + 0x7
#define EMV_GET_AMOUNT_ERROR 				EMV_ERR_BASE + 0x8
#define EMV_CARD_REJECTED 					EMV_ERR_BASE + 0x9
#define EMV_AIP_NOT_RECEIVED 				EMV_ERR_BASE + 0x10
#define EMV_AFL_NOT_RECEIVED 				EMV_ERR_BASE + 0x11
#define EMV_AFL_LEN_OUT_OF_RANGE 			EMV_ERR_BASE + 0x12
#define EMV_SFI_OUT_OF_RANGE 				EMV_ERR_BASE + 0x13
#define EMV_AFL_INCORRECT 					EMV_ERR_BASE + 0x14
#define EMV_EXP_DATE_INCORRECT 				EMV_ERR_BASE + 0x15
#define EMV_EFF_DATE_INCORRECT 				EMV_ERR_BASE + 0x16
#define EMV_ISS_COD_TBL_OUT_OF_RANGE 		EMV_ERR_BASE + 0x17
#define EMV_CRYPTOGRAM_TYPE_INCORRECT 		EMV_ERR_BASE + 0x18
#define EMV_PSE_NOT_SUPPORTED_BY_CARD		EMV_ERR_BASE + 0x19
#define EMV_USER_SELECTED_LANGUAGE			EMV_ERR_BASE + 0x20
#define EMV_SERVICE_NOT_ALLOWED 			EMV_ERR_BASE + 0x21
#define EMV_NO_TAG_FOUND 					EMV_ERR_BASE + 0x22
#define EMV_CARD_BLOCKED					EMV_ERR_BASE + 0x23
#define EMV_LEN_INCORRECT 					EMV_ERR_BASE + 0x24
#define CARD_COM_ERROR 						EMV_ERR_BASE + 0x25
#define EMV_TSC_NOT_INCREASED				EMV_ERR_BASE + 0x26
#define EMV_HASH_INCORRECT					EMV_ERR_BASE + 0x27
#define EMV_NO_ARC							EMV_ERR_BASE + 0x28
#define EMV_INVALID_ARC						EMV_ERR_BASE + 0x29
#define EMV_NO_ONLINE_COMM					EMV_ERR_BASE + 0x30
#define TRAN_TYPE_INCORRECT					EMV_ERR_BASE + 0x31

//CVM BASED ERRORS
#define CVM_ERR_BASE						ERR_BASE + 0x600
#define CVM_TYPE_UNKNOWN					CVM_ERR_BASE + 0x1
#define CVM_AIP_NOT_SUPPORTED				CVM_ERR_BASE + 0x2
#define CVM_TAG_8E_MISSING					CVM_ERR_BASE + 0x3
#define CVM_TAG_8E_FORMAT_ERROR				CVM_ERR_BASE + 0x4

#define CVM_CODE_IS_NOT_SUPPORTED			CVM_ERR_BASE + 0x5
#define CVM_COND_CODE_IS_NOT_SUPPORTED		CVM_ERR_BASE + 0x6
//#define CVM_TAG_8E_CVMRULE_MISSING			CVM_ERR_BASE + 7
#define NO_MORE_CVM			CVM_ERR_BASE + 0x7
#define PIN_BYPASSED_BEFORE					CVM_ERR_BASE + 0x8
//PUBLIC KEY LOAD
#define PK_LOAD_ERR_BASE					ERR_BASE + 0x700
#define PK_BUFFER_SIZE_TOO_BIG				PK_LOAD_ERR_BASE + 0x1
#define PK_FILE_WRITE_ERROR					PK_LOAD_ERR_BASE + 0x2
#define PK_HASH_ERROR						PK_LOAD_ERR_BASE + 0x3

//WARNING CODES
#define WARNING_CODES_BASE					ERR_BASE + 0x1000
#define EMV_APP_SELECTION_RETRY2			WARNING_CODES_BASE + 0x1
#define NO_CARD_HOLDER_CONFIRMATION			WARNING_CODES_BASE + 0x2
#define GET_ONLINE_PIN						WARNING_CODES_BASE + 0x3

#endif	/* _ERROR_H */
