//
//  AppDelegate.m
//  Sample
//

#import "AppDelegate.h"
#import <IDTech/IDTUtility.h>
#import <IDTech/IDT_UniPay.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>
#include <IOKit/usb/USBSpec.h>
#include <CommonCrypto/CommonDigest.h>

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    NSString *version = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    NSString *framework = [IDT_Device SDK_version ];
    
    [IDT_Device setDeviceType:IDT_DEVICE_UNIPAY_OSX_USB];
    [[IDT_Device sharedController] setDelegate:self];
    [[IDTechEMV sharedController] setDelegate:self];
    
    
    
    _window.title = [NSString stringWithFormat:@"UniPay Demo %@, Framework %@",version,framework];
    [emvLabel setStringValue: @""];
}

-(id) returnClass{
    return [IDT_Device class];
}

-(void) displayUpRet2:(NSString*) operation returnValue: (RETURN_CODE)rt
{
    
    
    NSString * str = [NSString stringWithFormat:
                      @"%@ ERROR: ID-\"%i\", message: %@.",
                      operation, rt, [[IDT_UniPay sharedController] device_getResponseCodeString:rt]];
    
    [self report:str];
    
    
}

-(void)appendMessageToResults:(NSString*)string{
    [self report:string];
}
- (void)report:(NSString*)string;
{
    [[tv textStorage] beginEditing];
    [[[tv textStorage] mutableString] appendString:string];
    [[[tv textStorage] mutableString] appendString:@"\n"];
    [[tv textStorage] endEditing];
    
    NSRange range;
    range = NSMakeRange ([[tv string] length], 0);
    
    [tv scrollRangeToVisible: range];
}

static bool waitForRemove = false;
- (void) emvTransactionMessage:(MESSAGE_Types)message{
    switch (message) {
        case MESSAGE_INSERT_CARD:
            [self appendMessageToResults:@"PLEASE INSERT OR SWIPE CARD"];
            [emvLabel setStringValue: @"PLEASE INSERT OR SWIPE CARD"];
            break;
        case MESSAGE_REMOVE_CARD:
            [self appendMessageToResults:@"PLEASE REMOVE CARD"];
            //lcdDisplay.text = @"PLEASE REMOVE CARD";
            waitForRemove = true;
            break;
        case MESSAGE_BAD_ICC:
            [self appendMessageToResults:@"CHIP READ ERROR-USE MSR"];
            [emvLabel setStringValue: @"CHIP READ ERROR-USE MSR"];
            break;
        case MESSAGE_TRANSACTION_CANCELLED:
            [self appendMessageToResults:@"TRANSACTION CANCELLED"];
            [emvLabel setStringValue: @"TRANSACTION CANCELLED"];
            break;
        case MESSAGE_FALLBACK_FAILED:
            [self appendMessageToResults:@"FALLBACK FAILED"];
            [emvLabel setStringValue:@"FALLBACK FAILED\nTRANSACTION CANCELLED" ];
            break;
        case MESSAGE_USE_CHIP_READER:
            [self appendMessageToResults:@"MUST USE ICC READER"];
            [emvLabel setStringValue: @"USE ICC READER\nPLEASE INSERT CARD"];
            break;
        case MESSAGE_PROCESSING:
            [self appendMessageToResults:@"PROCESSING"];
            [emvLabel setStringValue: @"PROCESSING..."];
            break;
        case MESSAGE_READY:
            [self appendMessageToResults:@"Ready"];
            [emvLabel setStringValue:@"Ready"];
            break;
        case MESSAGE_USE_MSR:
            [self appendMessageToResults:@"Please Swipe Card"];
            [emvLabel setStringValue: @"PLEASE SWIPE CARD"];
            break;
        case MESSAGE_NOT_ACCEPTED:
            [self appendMessageToResults:@"Not Accepted"];
            [emvLabel setStringValue:@"NOT ACCEPTED"];
            break;
            
            
            
            
            
        default:
            break;
    }

}



-(IBAction)openUSB:(id)sender{
    

    [[IDT_Device sharedController] attemptConnect];;
}


-(IBAction)close:(id)sender{
    [[IDT_Device sharedController] close];

}
-(IBAction) getTerminalFile:(id)sender{

    IDTech_TerminalData t = [[IDTechEMV sharedController] retrieveTerminalData];
    NSString* terminalCC = [NSString stringWithFormat:@"%4x",t.terminalCountryCode[0]*0x100+t.terminalCountryCode[1]];
    terminalCC = [terminalCC stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults: [NSString stringWithFormat:@"Terminal Country Code = %@", terminalCC]];
    
    
    NSString* merchantCC = [NSString stringWithFormat:@"%4x",t.merchantCategoryCode[0]*0x100+t.merchantCategoryCode[1]];
    merchantCC = [merchantCC stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults: [NSString stringWithFormat:@"Merchant Country Code = %@", merchantCC]];
    
    NSString* merchantID = [[NSString alloc] initWithBytes:&t.merchantID length:15 encoding:NSASCIIStringEncoding];
    [self appendMessageToResults: [NSString stringWithFormat:@"Merchant ID = %@", merchantID]];
    NSString* terminalID = [[NSString alloc] initWithBytes:&t.terminalID length:8 encoding:NSASCIIStringEncoding];
    [self appendMessageToResults: [NSString stringWithFormat:@"Terminal ID = %@", terminalID]];
    
    
    NSString* TACDefault = @"";
    
    TACDefault = [NSString stringWithFormat:@"%2x%2x%2x%2x%2x",t.defaultTACDefault[0],t.defaultTACDefault[1],t.defaultTACDefault[2],t.defaultTACDefault[3],t.defaultTACDefault[4]];
    TACDefault = [TACDefault stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults: [NSString stringWithFormat:@"TAC Default = %@", TACDefault]];
    
    
    
    NSString* TACDenial = @"";
    TACDenial = [NSString stringWithFormat:@"%2x%2x%2x%2x%2x",t.defaultTACDenial[0],t.defaultTACDenial[1],t.defaultTACDenial[2],t.defaultTACDenial[3],t.defaultTACDenial[4]];
    TACDenial = [TACDenial stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults: [NSString stringWithFormat:@"TAC Denial = %@", TACDenial]];
    
    
    NSString* TACOnline = @"";
    TACOnline = [NSString stringWithFormat:@"%2x%2x%2x%2x%2x",t.defaultTACOnline[0],t.defaultTACOnline[1],t.defaultTACOnline[2],t.defaultTACOnline[3],t.defaultTACOnline[4]];
    TACOnline = [TACOnline stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults: [NSString stringWithFormat:@"TAC TACOnline = %@", TACOnline]];
    
    NSString* terminalLocation = [[NSString alloc] initWithBytes:&t.terminalLocation length:36 encoding:NSASCIIStringEncoding];
    [self appendMessageToResults: [NSString stringWithFormat:@"Terminal Location = %@", terminalLocation]];
    
   
    
}


-(IBAction)createTerminalFile:(id)sender{
   

    IDTech_TerminalData t;
    t.terminalCountryCode[0] = 0x08;
    t.terminalCountryCode[1] = 0x40;
    t.merchantCategoryCode[0] = 0x12;
    t.merchantCategoryCode[1] = 0x34;
    t.merchantID[0] = 'I';
    t.merchantID[1] = 'D';
    t.merchantID[2] = ' ';
    t.merchantID[3] = 'T';
    t.merchantID[4] = 'E';
    t.merchantID[5] = 'C';
    t.merchantID[6] = 'H';
    t.merchantID[7] = ' ';
    t.merchantID[8] = 'R';
    t.merchantID[9] = '&';
    t.merchantID[10] = 'D';
    t.merchantID[11] = ' ';
    t.merchantID[12] = ' ';
    t.merchantID[13] = ' ';
    t.merchantID[14] = ' ';
    
    t.terminalID[0] = 0x38;
    t.terminalID[1] = 0x37;
    t.terminalID[2] = 0x36;
    t.terminalID[3] = 0x35;
    t.terminalID[4] = 0x34;
    t.terminalID[5] = 0x33;
    t.terminalID[6] = 0x32;
    t.terminalID[7] = 0x31;
    
    
    t.defaultTACDefault[0] = 0x00;
    t.defaultTACDefault[1] = 0x00;
    t.defaultTACDefault[2] = 0x80;
    t.defaultTACDefault[3] = 0x00;
    t.defaultTACDefault[4] = 0x00;
    t.defaultTACDenial[0] = 0x00;
    t.defaultTACDenial[1] = 0x00;
    t.defaultTACDenial[2] = 0x00;
    t.defaultTACDenial[3] = 0x00;
    t.defaultTACDenial[4] = 0x00;
    t.defaultTACOnline[0] = 0x00;
    t.defaultTACOnline[1] = 0x00;
    t.defaultTACOnline[2] = 0x00;
    t.defaultTACOnline[3] = 0x00;
    t.defaultTACOnline[4] = 0x00;
    
    t.useDefaultTACDefault = 0x00;
    t.useDefaultTACDenial = 0x00;
    t.useDefaultTACOnline = 0x00;
    
    t.terminalLocation[0] = '1';
    t.terminalLocation[1] = '0';
    t.terminalLocation[2] = '7';
    t.terminalLocation[3] = '2';
    t.terminalLocation[4] = '1';
    t.terminalLocation[5] = ' ';
    t.terminalLocation[6] = 'W';
    t.terminalLocation[7] = 'a';
    t.terminalLocation[8] = 'l';
    t.terminalLocation[9] = 'k';
    t.terminalLocation[10] = 'e';
    t.terminalLocation[11] = 'r';
    t.terminalLocation[12] = ' ';
    t.terminalLocation[13] = 'S';
    t.terminalLocation[14] = 't';
    t.terminalLocation[15] = '.';
    t.terminalLocation[16] = ' ';
    t.terminalLocation[17] = 'C';
    t.terminalLocation[18] = 'y';
    t.terminalLocation[19] = 'p';
    t.terminalLocation[20] = 'r';
    t.terminalLocation[21] = 'e';
    t.terminalLocation[22] = 's';
    t.terminalLocation[23] = 's';
    t.terminalLocation[24] = ',';
    t.terminalLocation[25] = ' ';
    t.terminalLocation[26] = 'C';
    t.terminalLocation[27] = 'A';
    t.terminalLocation[28] = ',';
    t.terminalLocation[29] = ' ';
    t.terminalLocation[30] = 'U';
    t.terminalLocation[31] = 'S';
    t.terminalLocation[32] = 'A';
    t.terminalLocation[33] = ' ';
    t.terminalLocation[34] = ' ';
    t.terminalLocation[35] = ' ';
    
    
    [[IDTechEMV sharedController] setTerminalData:t];
    [emvLabel setStringValue:@"Terminal Settings Created"];
}

-(IBAction) createAIDFile:(id)sender{

    IDTech_ApplicationID a;
    
    
    a.acquirerIdentifier[0] = 0x00;
    a.acquirerIdentifier[1] = 0x00;
    a.acquirerIdentifier[2] = 0x00;
    a.acquirerIdentifier[3] = 0x00;
    a.acquirerIdentifier[4] = 0x00;
    a.acquirerIdentifier[5] = 0x00;
    a.aid[0] = 0xa0;
    a.aid[1] = 0x00;
    a.aid[2] = 0x00;
    a.aid[3] = 0x00;
    a.aid[4] = 0x03;
    a.aid[5] = 0x10;
    a.aid[6] = 0x10;
    a.aid[7] = 0x00;
    a.aid[8] = 0x00;
    a.aid[9] = 0x00;
    a.aid[10] = 0x00;
    a.aid[11] = 0x00;
    a.aid[12] = 0x00;
    a.aid[13] = 0x00;
    a.aid[14] = 0x00;
    a.aid[15] = 0x00;
    a.aidLen = 0x07;
    a.applicationSelectionIndicator = 0x01;
    a.applicationVersionNumber[0] = 0x00;
    a.applicationVersionNumber[1] = 0x01;
    
    
    a.transactionCurrencyCode[0]=0x08;
    a.transactionCurrencyCode[1]=0x40;
    a.transactionCurrencyExponent = 0x02;
    a.tacDenial[0] = 0x00;
    a.tacDenial[1] = 0x00;
    a.tacDenial[2] = 0x00;
    a.tacDenial[3] = 0x00;
    a.tacDenial[4] = 0x00;
    a.useTACDenial = 0x00;
    
    a.tacDefault[0] = 0x00;
    a.tacDefault[1] = 0x00;
    a.tacDefault[2] = 0x00;
    a.tacDefault[3] = 0x00;
    a.tacDefault[4] = 0x00;
    a.useTACDefault = 0x00;
    
    a.tacOnline[0] = 0x00;
    a.tacOnline[1] = 0x00;
    a.tacOnline[2] = 0x00;
    a.tacOnline[3] = 0x00;
    a.tacOnline[4] = 0x00;
    a.useTACOnline = 0x00;
    
    [[IDTechEMV sharedController] setApplicationData:a];
    
    
    [emvLabel setStringValue:@"AID for a0000000031010 created"];
}
-(IBAction) getAIDFile:(id)sender{

    IDTech_ApplicationID t = [[IDTechEMV sharedController] retrieveApplicationData:@"a0000000031010"];
    if (t.aidLen == 0) {
        [self appendMessageToResults:@"AID File Not Returned - Try Again"];
        return;
    }
    
    [self appendMessageToResults:[[NSString stringWithFormat:@"CurrencyCode:%4x",t.transactionCurrencyCode[0]*0x100+t.transactionCurrencyCode[1]] stringByReplacingOccurrencesOfString:@" " withString:@"0"]];
    
    
    NSData* _aaid = [NSData dataWithBytes:&t.acquirerIdentifier length:6];
    NSString *aAquirerID = [[[[_aaid description] stringByReplacingOccurrencesOfString:@"<" withString:@""]
                             stringByReplacingOccurrencesOfString:@">" withString:@""]
                            stringByReplacingOccurrencesOfString:@" " withString:@""];
    [self appendMessageToResults:[NSString stringWithFormat:@"AquirerID: %@",aAquirerID]];
    
    NSData* _aid = [NSData dataWithBytes:&t.aid length:t.aidLen];
    NSString* aAID = [[[[_aid description] stringByReplacingOccurrencesOfString:@"<" withString:@""]
                       stringByReplacingOccurrencesOfString:@">" withString:@""]
                      stringByReplacingOccurrencesOfString:@" " withString:@""];
    [self appendMessageToResults:[NSString stringWithFormat:@"AID: %@",aAID]];
    
    NSString* aAIDVersion = [NSString stringWithFormat:@"%4x",t.applicationVersionNumber[0]*0x100+t.applicationVersionNumber[1]];
    aAIDVersion = [aAIDVersion stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults:[NSString stringWithFormat:@"AID Version: %@",aAIDVersion]];
    
    NSString* aCurrencyExp = [NSString stringWithFormat:@"%2x",t.transactionCurrencyExponent];
    aCurrencyExp = [aCurrencyExp stringByReplacingOccurrencesOfString:@" " withString:@"0"];
    [self appendMessageToResults:[NSString stringWithFormat:@"Currency Exp: %@",aCurrencyExp]];
    
    
    NSData* _tac = [NSData dataWithBytes:&t.tacDenial length:5];
    NSString* aTACDenial = @"";
    aTACDenial = [[[[_tac description] stringByReplacingOccurrencesOfString:@"<" withString:@""]
                   stringByReplacingOccurrencesOfString:@">" withString:@""]
                  stringByReplacingOccurrencesOfString:@" " withString:@""];
    [self appendMessageToResults:[NSString stringWithFormat:@"TAC Denial: %@",aTACDenial]];
    
    NSData* _tacOnline = [NSData dataWithBytes:&t.tacOnline length:5];
    NSString* aTACOnline = @"";
    aTACOnline = [[[[_tacOnline description] stringByReplacingOccurrencesOfString:@"<" withString:@""]
                   stringByReplacingOccurrencesOfString:@">" withString:@""]
                  stringByReplacingOccurrencesOfString:@" " withString:@""];
    [self appendMessageToResults:[NSString stringWithFormat:@"TAC Online: %@",aTACOnline]];
    
    
    NSData* _tacDefault = [NSData dataWithBytes:&t.tacDefault length:5];
    NSString* aTACDefault = @"";
    aTACDefault = [[[[_tacDefault description] stringByReplacingOccurrencesOfString:@"<" withString:@""]
                    stringByReplacingOccurrencesOfString:@">" withString:@""]
                   stringByReplacingOccurrencesOfString:@" " withString:@""];
    [self appendMessageToResults:[NSString stringWithFormat:@"TAC Default: %@",aTACDefault]];
   
}
-(IBAction) removeAIDFile:(id)sender{

    [[IDTechEMV sharedController] removeApplicationData:@"a0000000031010"];
    [self appendMessageToResults:@"Removed AID a0000000031010"];
    
}
-(IBAction) getAIDFList:(id)sender{

    NSArray *dict =[[IDTechEMV sharedController] retrieveAIDList];
    
    NSLog(@"array = %@",dict.description);
    [self appendMessageToResults:[NSString stringWithFormat:@"array = %@", dict.description]];
}



-(IBAction) iccStatus:(id)sender{
    ICCReaderStatus* dict;
   RETURN_CODE rt = [[IDT_Device sharedController] getICCReaderStatus:&dict];
    NSData *data = [NSData dataWithBytes:&dict[0] length:sizeof(dict)];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
         
        [self report: [NSString stringWithFormat:@"iccStatus File: %@",data.description]];
    }
    else{
        [self displayUpRet2: @"iccStatus File" returnValue: rt];
        return;
        
    }


    NSLog(@"icc power = %@",dict->iccPower?@"True":@"False");
    NSLog(@"cardSeated = %@",dict->cardSeated?@"True":@"False");
    NSLog(@"latchClosed = %@",dict->latchClosed?@"True":@"False");
    NSLog(@"cardPresent = %@",dict->cardPresent?@"True":@"False");
    NSLog(@"magneticDataPresent = %@",dict->magneticDataPresent?@"True":@"False");
}





- (void) emvTransactionData:(IDTEMVData*)emvData errorCode:(int)error performReversal:(BOOL)reversal{
    NSLog(@"Dictionary = %@",emvData.unencryptedTags.description);
    NSLog(@"Result code = %2X, Error Code %4X",emvData.resultCode,error);
    if (emvData.resultCode == 2) {
        [emvLabel setStringValue: @"RESULT: GO ONLINE - COMPLETION REQUIRED"];
    }
    if (emvData.resultCode == 0) {
        [emvLabel setStringValue: @"TRANSACTION SUCCCESS"];
    }
    
    [self appendMessageToResults:[NSString stringWithFormat:@"Dictionary = %@",emvData.unencryptedTags.description]];
    [self appendMessageToResults:[NSString stringWithFormat:@"Result code = %2X, Error Code %4X",emvData.resultCode,error]];
}

-(void) clearMemory:(unsigned char*)loc length:(int)len{
    if (len < 1) {
        return;
    }
    for (int x = 0; x < len; x ++) {
        loc[x] = 0x00;
    }
}

-(IBAction)completeEMV:(id)sender{
    


    NSMutableDictionary* dict = [NSMutableDictionary new];
    
    [dict setObject:[self hexToData:@"3030"] forKey:@"8a"];
    
    NSDictionary* myDict;
    myDict = nil;
    [[IDTechEMV sharedController]  completeOnlineEMVTransaction:EMV_COMPLETION_RESULT_ACCEPTED resultCode:@"00" issuerAuthenticationData:nil issuerScripts:nil];
}






-(IBAction)startEMV:(id)sender{
/*
    [[IDT_UniPay sharedController] icc_powerOnICC:nil];
    double current = [NSDate timeIntervalSinceReferenceDate];
    NSMutableArray *ar = [[NSMutableArray alloc] init];
    for (int x = 0; x < 30; x++) {
        [ar addObject:[self hexToData:@"0d00a4040007a000000003101000"]];

    [[IDT_UniPay sharedController] icc_exchangeMultiAPDU:ar response:nil];
    NSLog(@"Total Time = %f", [NSDate timeIntervalSinceReferenceDate] - current);
    //[[IDT_UniPay sharedController] icc_powerOffICC:nil];
    return;
 }
 */
    NSMutableDictionary* tags;
    [[IDTechEMV sharedController] startEMVTransaction:2.95 timeout:30 transactionType:0 additionalTags:tags];
    
}

-(void) deviceConnected{
    [connectedLabel setStringValue:@"DEVICE CONNECTED"];
}
-(void) deviceDisconnected{
    [connectedLabel setStringValue:@"DEVICE DISCONNECTED"];
}

- (void) dataInOutMonitor:(NSData*)data  incoming:(BOOL)isIncoming{
    NSLog(@"Received Data====================================");
     
    [self report:[NSString stringWithFormat:@"\n%@: %@",isIncoming?@"IN":@"OUT",data.description]];
}



- (void) swipeMSRData:(IDTMSRData*)cardData{
    switch (cardData.event) {
        case EVENT_MSR_CARD_DATA:
        {
            
            switch (cardData.captureEncodeType) {
                case CAPTURE_ENCODE_TYPE_ISOABA:
                    
                    [self report:[NSString stringWithFormat:@"\nEncode Type: %@", @"ISO/ABA"]];
                    break;
                case CAPTURE_ENCODE_TYPE_AAMVA:
                    [self report:[NSString stringWithFormat:@"\nEncode Type: %@",  @"AA/MVA"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Other:
                    [self report:[NSString stringWithFormat:@"\nEncode Type: %@",  @"Other"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Raw:
                    [self report:[NSString stringWithFormat:@"\nEncode Type: %@",  @"Raw"]];
                    break;
                    
                default:
                    [self report:[NSString stringWithFormat:@"\nEncode Type: %@",  @"UNKNOWN"]];
                    
                    break;
            }
            switch (cardData.captureEncryptType) {
                case CAPTURE_ENCRYPT_TYPE_AES:
                    [self report:[NSString stringWithFormat:@"\nEncryption Type: %@", @"AES"]];
                    break;
                case CAPTURE_ENCRYPT_TYPE_TDES:
                    [self report:[NSString stringWithFormat:@"\nEncryption Type: %@",  @"TDES"]];
                    break;
                
                    
                default:
                    [self report:[NSString stringWithFormat:@"\nEncryption Type: %@",  @"UNKNOWN"]];
                    
                    break;
            }
            if (cardData.cardData.length > 0) [self report:[NSString stringWithFormat:@"\nCard Data: %@",  cardData.cardData]];
            if (cardData.track1Length > 0) [self report:[NSString stringWithFormat:@"\nTrack 1(%i): %@", cardData.track1Length, cardData.track1]];
            if (cardData.track2Length > 0) [self report: [NSString stringWithFormat:@"\nTrack 2(%i): %@",  cardData.track2Length, cardData.track2]];
            if (cardData.track3Length > 0) [self report:[NSString stringWithFormat:@"\nTrack 3(%i): %@",  cardData.track3Length, cardData.track3]];
            if (cardData.encTrack1.description.length > 0) [self report:[NSString stringWithFormat:@"\nEncoded Track 1: %@",  cardData.encTrack1.description]];
            if (cardData.encTrack2.description.length > 0) [self report:[NSString stringWithFormat:@"\nEncoded Track 2: %@",  cardData.encTrack2.description]];
            if (cardData.encTrack3.description.length > 0) [self report:[NSString stringWithFormat:@"\nEncoded Track 3: %@",  cardData.encTrack3.description]];
            if (cardData.hashTrack1.description.length > 0) [self report:[NSString stringWithFormat:@"\nHash Track 1: %@",  cardData.hashTrack1.description]];
            if (cardData.hashTrack2.description.length > 0) [self report:[NSString stringWithFormat:@"\nHash Track 2: %@",  cardData.hashTrack2.description]];
            if (cardData.hashTrack3.description.length > 0) [self report:[NSString stringWithFormat:@"\nHash Track 3: %@",  cardData.hashTrack3.description]];
            if (cardData.KSN.description.length > 0) [self report:[NSString stringWithFormat:@"\nKSN: %@",  cardData.KSN.description]];
            if (cardData.sessionID.description.length > 0) [self report:[NSString stringWithFormat:@"\nSessionID: %@",  cardData.sessionID.description]];
            if (cardData.RSN.length > 0) [self report:[NSString stringWithFormat:@"\nReader Serial Number: %@",  cardData.RSN]];
            [self report:[NSString stringWithFormat:@"\nRead Status: %2X",  cardData.readStatus]];

            return;
        }
            break;
            
        case EVENT_MSR_CANCEL_KEY:
        {
            [self report:[NSString stringWithFormat:@"\n(Event) MSR Cancel Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_BACKSPACE_KEY:
        {
            [self report:[NSString stringWithFormat:@"\n(Event) MSR Backspace Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_ENTER_KEY:
        {
            [self report:[NSString stringWithFormat:@"\n(Event) MSR Enter Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_UNKNOWN:
        {
            [self report:[NSString stringWithFormat:@"\n(Event) MSR unknown event, data: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        default:
            break;
    }
}

-(IBAction) sendAPDU:(id)sender{
    NSData *dataBuffer = nil;
    NSString *valueEntered = [apdu.cell title];
 
    dataBuffer = [self hexStringToBytes:valueEntered];

    APDUResponse* response;
   
    [[IDT_Device sharedController] exchangeAPDU:dataBuffer encrypted:false ksn:nil response:&response];
    if (response == nil) {
        return;
    }
    
}



-(IBAction) msrON:(id)sender{

    RETURN_CODE rt = [[IDT_UniPay sharedController] msr_startMSRSwipe:0];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"msrON: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"msrON" returnValue: rt];
        return;
    }
}
-(IBAction) msrOff:(id)sender{

    RETURN_CODE rt = [[IDT_UniPay sharedController] msr_cancelMSRSwipe];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"msrOFF: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"msrOFF" returnValue: rt];
        return;
    }


}
-(IBAction) powerOnICC:(id)sender{

    NSData *retVal = nil;
    RETURN_CODE rt = [[IDT_UniPay sharedController] icc_powerOnICC:&retVal ];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"powerOnICC: %@",retVal.description]];
    }
    else{
        [self displayUpRet2: @"powerOnICC" returnValue: rt];
        return;
    }
}

-(IBAction) powerOffICC:(id)sender{
    NSString *retVal = nil;
    RETURN_CODE rt = [[IDT_UniPay sharedController] icc_powerOffICC:&retVal ];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"powerOffICC: %@",retVal.description]];
    }
    else{
        [self displayUpRet2: @"powerOffICC" returnValue: rt];
        return;
    }

    
}
-(IBAction) requestPIN:(id)sender{
    RETURN_CODE rt = [[IDT_UniPayII sharedController] pin_getEncryptedPIN:@"1234567890123456" keyType:PIN_KEY_TDES_DUKPT_extp message:@"Please Enter PIN"];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"requestPIN: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"requestPIN" returnValue: rt];
        return;
    }
    
}

-(IBAction) getBatteryLevel:(id)sender{
    NSString *result;
    RETURN_CODE rt = [[IDT_UniPay sharedController]  device_getBatteryVoltage:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"Get Battery Voltage: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Battery Voltage" returnValue: rt];
        return;
    }
}


- (IBAction) f_getFirm:(id)sender
{    NSString *result;
    RETURN_CODE rt = [[IDT_UniPay sharedController]  device_getFirmwareVersion:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"Get Firmware: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Firmware" returnValue: rt];
        return;
    }
 
}
- (IBAction) f_getModelNumber:(id)sender
{
    NSString* result;
    RETURN_CODE rt = [[IDT_UniPay sharedController] config_getModelNumber:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"Get Model No: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Model No" returnValue: rt];
        return;
    }

}

- (IBAction) f_getSerialNumber:(id)sender
{
    NSString* result;
    RETURN_CODE rt = [[IDT_UniPay sharedController] config_getSerialNumber:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        
        [self report: [NSString stringWithFormat:@"Get Serial No: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Model No" returnValue: rt];
        return;
    }

}

-(unsigned int) char2hex:(char)c{
	
	switch (c) {
		case '0' ... '9': return c - '0';
		case 'a' ... 'f': return c - 'a' + 10;
		case 'A' ... 'F': return c - 'A' + 10;
		default: return -1;
	}
}

- (NSData *)hexToData:(NSString*)str {   //Example - Pass string that contains characters "30313233", and it will return a data object containing ascii characters "0123"
	if ([str length] == 0) {
		return nil;
	}
	
	unsigned stringIndex=0, resultIndex=0, max=(int)[str length];
	NSMutableData* result = [NSMutableData dataWithLength:(max + 1)/2];
	unsigned char* bytes = [result mutableBytes];
	
	unsigned num_nibbles = 0;
	unsigned char byte_value = 0;
	
	for (stringIndex = 0; stringIndex < max; stringIndex++) {
		unsigned int val = [self char2hex:[str characterAtIndex:stringIndex]];
		
		num_nibbles++;
		byte_value = byte_value * 16 + (unsigned char)val;
		if (! (num_nibbles % 2)) {
			bytes[resultIndex++] = byte_value;
			byte_value = 0;
		}
	}
	
	
	//final nibble
	if (num_nibbles % 2) {
		bytes[resultIndex++] = byte_value;
	}
	
	[result setLength:resultIndex];
	
	return result;
}

-(BOOL) isValidChar:(Byte)byChar{
    if ( ((byChar>(Byte)0x2F) && (byChar<(Byte)0x3A)) //'0' - '9'
        || ((byChar>(Byte)0x40) && (byChar<(Byte)0x47)) //'A' - 'F'
        || ((byChar>(Byte)0x60) && (byChar<(Byte)0x67)) //'a' - 'f'
        )
    {
        return YES;
    }
    return NO;
}


-(NSData*) hexStringToBytes:(NSString*)hexString{
    if (hexString.length<1) {
        return nil;
    }
    
    char tmpCh[1024] = {0};
    int count = 0;
    for (int k=0; k<hexString.length;k++) {
        char c = [hexString characterAtIndex:k];
        if (c == (char)0x20) {
            continue;
        }
        tmpCh[count] = c;
        count++;
    }
    tmpCh[count] = 0;
    
    if (count % 2) {
        return nil;
    }
    
    NSString *temp = [[NSString alloc] initWithUTF8String:tmpCh];
    int m = temp.length % 2;
    if (m>0) {
        return nil;
    }
    
    NSMutableData *result = [[NSMutableData alloc] init];
    unsigned char byte;
    char hexChars[3] = {0};
    for (int i=0; i < (temp.length/2); i++) {
        hexChars[0] = [temp characterAtIndex:i*2];
        hexChars[1] = [temp characterAtIndex:i*2+1];
        
        if (![self isValidChar:hexChars[0]] || ![self isValidChar:hexChars[1]]) {
            return nil;
        }
        byte = strtol(hexChars, NULL, 16);
        [result appendBytes:&byte length:1];
    }
    return [NSData dataWithData:result];
}

@end
